/****************************************************************************
*                          PHASES OF THE MOON in C                          *
*                                                                           *
*           Translated from the Basic program, ver 1.0.0 (March             *
*           8, 1987) by Daniel P. Franco, to Turbo-C by:                    *
*                              Dylan MacHattie                              *
****************************************************************************/

#include <stdio.h>
#include <dos.h>
#include <math.h>

#define PI      3.141592653589793
#define SOLY    365.2421875       /* Solar year */

void syntax_err(void);

struct dat {
    int month;
    int year;
} show;

void main(int argc, char *argv[])
{
    double d, k, t, t2, t3, r, sma, mma, f, jd, z, frack, frac;
    double a, b, e, dom, y, m, tothours, day, totmin, hour, sec, min, junk;
    char *phase[4] = {"New moon","First quarter","Full moon","Third quarter"};
    int i, leap=0, last, first;

    puts(" Moon Phase Calculator");

    if ((argc < 3)||(argc > 4)) syntax_err();

    show.month = atoi(argv[1]);
    if ((show.month<1)||(show.month>12)) syntax_err();
    if ((show.year = atoi(argv[2])) > 9999) syntax_err();
    if (argc == 4)
    {
        if ((last=atoi(argv[3])) > 9) syntax_err();
    }
    else last = 1;

    last = (last + show.month - 1)%12 + 1;
    first = (show.month + 10)%12 + 1;
    if (first==12) y = (double)(show.year-1);
    else y = (double)show.year;

    if ((!(show.year%400))||
            ((!(show.year%4))&&(show.year%100))) leap = 1;

/* decimal year calculation */
    switch (first)
    {
        case 1:     d = 15.5; break;
        case 2:     d = 45+.5*leap; break;
        case 3:     d = 74.5+leap; break;
        case 4:     d = 105+leap; break;
        case 5:     d = 135.5+leap; break;
        case 6:     d = 166+leap; break;
        case 7:     d = 196.5+leap; break;
        case 8:     d = 227.5+leap; break;
        case 9:     d = 258+leap; break;
        case 10:    d = 228.5+leap; break;
        case 11:    d = 319+leap; break;
        case 12:    d = 349.5+leap;
    }
    k = (y - 1900 + d/SOLY) * 12.3685;
    if (modf(k,&junk)<0.5) k = floor(k);
    else k = ceil(k);

    do
    {
        t = k/1236.85;
        t2 = pow(t,2);
        t3 = pow(t,3);
        r = PI/180;

/* sun mean anomaly */
        sma = r * fmod((359.2242+(29.10535608*k)-(.0000333*t2)-(.00000347*t3)),360);

/* moon mean anomaly */
        mma = r * fmod((306.0253+(385.81691806*k)+(.0107306*t2)+(.00001236*t3)),360);

/* moon's argument of latitude */
        f = r * fmod((21.2964+(390.67050646*k)-(.0016528*t2)-(.00000239*t3)),360);

/* mean phase of the moon */
        jd = 2415020.75933+(29.53058868*k)+(.0001178*t2)-(1.55E-07*t3)
            +(.00033*sin((r*166.56)+(r*t*132.87)-(r*t2*.009173)));

/* true phase corrections for new and full moon */
        frack = modf(k,&junk);
        if ((frack==0)||(frack==0.5)||(frack==-.5))
        {
            jd=jd+((.1734-.000393*t)*sin(sma))+(.0021*sin(2*sma))-(.4068*sin(mma))
                +(.0161*sin(2*mma))-(.0004*sin(3*mma))+(.0104*sin(2*f))-(.0051*sin(sma+mma))
                -(.0074*sin(sma-mma))+(.0004*sin((2*f)+sma))-(.0004*sin((2*f)-sma))
                -(6.000001E-04*sin((2*f)+mma))+(.001*sin((2*f)-mma))+.0005*sin(sma+(2*mma));
        }

/* true phase corrections for first and third quarter */
        else
        {
            jd=jd+(.1721-.0004*t)*sin(sma)+.0021*sin(2*sma)-.628*sin(mma)+.0089*sin(2*mma)
            -.0004*sin(3*mma)+.0079*sin(2*f)-.0119*sin(sma+mma)-.0047*sin(sma-mma)
            +.0003*sin(2*f+sma)-.0004*sin(2*f-sma)-6.000001E-04*sin(2*f+mma)
            +.0021*sin(2*f-mma)+.0003*sin(sma+2*mma)+.0004*sin(sma-2*mma)
            -.0003*sin(2*sma-mma);
        }

/* additional first quarter correction */
        if (((k>=0)&&(frack==0.25))||((k<0)&&(frack==-0.75)))
        {
            jd=jd+.0028-.0004*cos(sma)+.0003*cos(mma);
        }

/* additional third quarter correction */
        else
        {
            if (((k>=0)&&(frack==0.75))||((k<0)&&(frack==-0.25)))
            {
                jd=jd-.0028+.0004*cos(sma)-.0003*cos(mma);
            }
        }

/* calendar date calculation */
        jd=jd+.5;
        z=floor(jd);
        frac=modf(jd,&junk);
        if (z<2299161) a = z;
        else
        {
            a = z + 1 + floor((z-1867216.25)/(100*SOLY)) - floor((z-1867216.25)/(400*SOLY));
        }
        b = a + 1524 - floor(365.25 * floor((a + 1401.9)/365.25));
        e = floor(b/30.6001);
        dom = b - floor(30.6001 * e) + frac;
        if (e<13.5) m = e - 1;
        else m = e - 13;
        if (m>2.5) y = floor((a+1401.9)/365.25) - 4716;
        else y = floor((a+1401.9)/365.25) - 4715;
        tothours = 24 * modf(dom,&day);
        totmin = 60 * modf(tothours,&hour);
        sec = 60 * modf(totmin,&min);

        i = ((int)((frack+1)*4))%4;
        if (((int)m != first)&&((int)m != last))
        {
            printf(" %2d/%02d/%d %2d:%02d:%02d UT  %s\n", (int) m, (int) day,
                    (int) y, (int) hour, (int) min, (int) sec, phase[i]);
        }
        k = k+0.25;
    } while ((int)m != last);
    return;
}
void syntax_err(void)
{
    puts(" Command line syntax:    moonph MM YYYY [N]");
    puts("    where:  MM is the first month to display (1=Jan, 12=Dec)");
    puts("            YYYY is the starting year");
    puts("            N is the number (1 to 9) of months to display (default 1)");
    exit(0);
}
